==> nuxeo-automation-server-10.2-sources.jar: org/nuxeo/ecm/automation/server/AutomationServerComponent.java <==
/*
 * (C) Copyright 2006-2017 Nuxeo (http://nuxeo.com/) and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     bstefanescu
 *     vpasquier
 *     slacoin
 */
package org.nuxeo.ecm.automation.server;

import java.security.Principal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;

import org.nuxeo.ecm.automation.core.Constants;
import org.nuxeo.ecm.automation.io.services.IOComponent;
import org.nuxeo.ecm.core.api.NuxeoPrincipal;
import org.nuxeo.runtime.api.Framework;
import org.nuxeo.runtime.model.ComponentContext;
import org.nuxeo.runtime.model.ComponentInstance;
import org.nuxeo.runtime.model.DefaultComponent;

/**
 * @author <a href="mailto:bs@nuxeo.com">Bogdan Stefanescu</a>
 */
public class AutomationServerComponent extends DefaultComponent implements AutomationServer {

    protected static final String XP_BINDINGS = "bindings";

    protected static final String IOCOMPONENT_NAME = "org.nuxeo.ecm.automation.io.services.IOComponent";

    protected IOComponent ioComponent;

    private static final String XP_MARSHALLER = "marshallers";

    protected Map<String, RestBinding> bindings;

    protected static final String XP_CODECS = "codecs";

    protected volatile Map<String, RestBinding> lookup;

    protected List<Class<? extends MessageBodyWriter<?>>> writers;

    protected List<Class<? extends MessageBodyReader<?>>> readers;

    @Override
    public void activate(ComponentContext context) {
        bindings = new HashMap<>();
        writers = new ArrayList<>();
        readers = new ArrayList<>();
        ioComponent = ((IOComponent) Framework.getRuntime().getComponentInstance(IOCOMPONENT_NAME).getInstance());
    }

    @Override
    public void deactivate(ComponentContext context) {
        bindings = null;
    }

    @Override
    public void registerContribution(Object contribution, String extensionPoint, ComponentInstance contributor) {
        if (XP_BINDINGS.equals(extensionPoint)) {
            RestBinding binding = (RestBinding) contribution;
            addBinding(binding);
        } else if (XP_MARSHALLER.equals(extensionPoint)) {
            MarshallerDescriptor marshaller = (MarshallerDescriptor) contribution;
            writers.addAll(marshaller.getWriters());
            readers.addAll(marshaller.getReaders());
        } else if (XP_CODECS.equals(extensionPoint)) {
            ioComponent.registerContribution(contribution, extensionPoint, contributor);
        }
    }

    @Override
    public void unregisterContribution(Object contribution, String extensionPoint, ComponentInstance contributor) {
        if (XP_BINDINGS.equals(extensionPoint)) {
            RestBinding binding = (RestBinding) contribution;
            removeBinding(binding);
        } else if (XP_CODECS.equals(extensionPoint)) {
            ioComponent.unregisterContribution(contribution, extensionPoint, contributor);
        }
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (AutomationServer.class.isAssignableFrom(adapter)) {
            return adapter.cast(this);
        }
        return null;
    }

    @Override
    public RestBinding getOperationBinding(String name) {
        return lookup().get(name);
    }

    @Override
    public RestBinding getChainBinding(String name) {
        return lookup().get(Constants.CHAIN_ID_PREFIX + name);
    }

    @Override
    public RestBinding[] getBindings() {
        Map<String, RestBinding> map = lookup();
        return map.values().toArray(new RestBinding[map.size()]);
    }

    protected String getBindingKey(RestBinding binding) {
        return binding.isChain() ? Constants.CHAIN_ID_PREFIX + binding.getName() : binding.getName();
    }

    @Override
    public synchronized void addBinding(RestBinding binding) {
        String key = getBindingKey(binding);
        bindings.put(key, binding);
        lookup = null;
    }

    @Override
    public synchronized RestBinding removeBinding(RestBinding binding) {
        RestBinding result = bindings.remove(getBindingKey(binding));
        lookup = null;
        return result;
    }

    @Override
    public boolean accept(String name, boolean isChain, HttpServletRequest req) {
        if (isChain) {
            name = Constants.CHAIN_ID_PREFIX + name;
        }
        RestBinding binding = lookup().get(name);
        if (binding != null) {
            if (binding.isDisabled()) {
                return false;
            }
            if (binding.isSecure()) {
                if (!req.isSecure()) {
                    return false;
                }
            }
            Principal principal = req.getUserPrincipal();

            if (binding.isAdministrator() || binding.hasGroups()) {
                if (principal instanceof NuxeoPrincipal) {
                    NuxeoPrincipal np = (NuxeoPrincipal) principal;
                    if (binding.isAdministrator() && np.isAdministrator()) {
                        return true;
                    }
                    if (binding.hasGroups()) {
                        for (String group : binding.getGroups()) {
                            if (np.isMemberOf(group)) {
                                return true;
                            }
                        }
                    }
                }
                return false;
            }
        }
        return true;
    }

    private Map<String, RestBinding> lookup() {
        Map<String, RestBinding> _lookup = lookup;
        if (_lookup == null) {
            synchronized (this) {
                lookup = new HashMap<>(bindings);
                _lookup = lookup;
            }
        }
        return _lookup;
    }

    @Override
    public List<Class<? extends MessageBodyWriter<?>>> getWriters() {
        return writers;
    }

    @Override
    public List<Class<? extends MessageBodyReader<?>>> getReaders() {
        return readers;
    }

}

==> nuxeo-automation-server-11.1-20200126.055008-338-sources.jar: org/nuxeo/ecm/automation/server/AutomationServerComponent.java <==
/*
 * (C) Copyright 2006-2017 Nuxeo (http://nuxeo.com/) and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 *     bstefanescu
 *     vpasquier
 *     slacoin
 */
package org.nuxeo.ecm.automation.server;

import java.security.Principal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;

import org.nuxeo.ecm.automation.core.Constants;
import org.nuxeo.ecm.core.api.NuxeoPrincipal;
import org.nuxeo.runtime.model.ComponentContext;
import org.nuxeo.runtime.model.DefaultComponent;

/**
 * @author <a href="mailto:bs@nuxeo.com">Bogdan Stefanescu</a>
 */
public class AutomationServerComponent extends DefaultComponent implements AutomationServer {

    /**
     * @since 10.3
     */
    public static final String XP_BINDINGS = "bindings";

    /**
     * @since 10.3
     */
    public static final String XP_MARSHALLER = "marshallers";

    protected List<Class<? extends MessageBodyWriter<?>>> writers = new ArrayList<>();

    protected List<Class<? extends MessageBodyReader<?>>> readers = new ArrayList<>();

    @Override
    public void start(ComponentContext context) {
        super.start(context);
        List<MarshallerDescriptor> marshallers = getDescriptors(XP_MARSHALLER);
        marshallers.forEach(m -> {
            writers.addAll(m.writers);
            readers.addAll(m.readers);
        });
    }

    @Override
    public void stop(ComponentContext context) throws InterruptedException {
        super.stop(context);
        writers.clear();
        readers.clear();
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (AutomationServer.class.isAssignableFrom(adapter)) {
            return adapter.cast(this);
        }
        return null;
    }

    @Override
    public RestBinding getOperationBinding(String name) {
        return getDescriptor(XP_BINDINGS, name);
    }

    @Override
    public RestBinding getChainBinding(String name) {
        return getDescriptor(XP_BINDINGS, Constants.CHAIN_ID_PREFIX + name);
    }

    @Override
    public RestBinding[] getBindings() {
        List<RestBinding> descriptors = getDescriptors(XP_BINDINGS);
        return descriptors.toArray(new RestBinding[0]);
    }

    @Override
    public synchronized void addBinding(RestBinding binding) {
        register(XP_BINDINGS, binding);
    }

    @Override
    public synchronized RestBinding removeBinding(RestBinding binding) {
        return unregister(XP_BINDINGS, binding) ? binding : null;
    }

    @Override
    public boolean accept(String name, boolean isChain, HttpServletRequest req) {
        if (isChain) {
            name = Constants.CHAIN_ID_PREFIX + name;
        }
        RestBinding binding = getDescriptor(XP_BINDINGS, name);
        if (binding != null) {
            if (binding.isDisabled) {
                return false;
            }
            if (binding.isSecure && !req.isSecure()) {
                return false;
            }
            Principal principal = req.getUserPrincipal();

            if (binding.isAdministrator || binding.hasGroups()) {
                if (principal instanceof NuxeoPrincipal) {
                    NuxeoPrincipal np = (NuxeoPrincipal) principal;
                    if (binding.isAdministrator && np.isAdministrator()) {
                        return true;
                    }
                    if (binding.hasGroups()) {
                        for (String group : binding.groups) {
                            if (np.isMemberOf(group)) {
                                return true;
                            }
                        }
                    }
                }
                return false;
            }
        }
        return true;
    }

    @Override
    public List<Class<? extends MessageBodyWriter<?>>> getWriters() {
        return writers;
    }

    @Override
    public List<Class<? extends MessageBodyReader<?>>> getReaders() {
        return readers;
    }

}
